import { asyncDelay, asyncCallNetworkApiGet, asyncGetSpriteFrimeByUrl, buttonOnClick,
    showLoading, hideLoading, loadOnlineCourseWare, showTips } from "../script/util";
import { MyCocosSceneComponent } from "../script/MyCocosSceneComponent";

const { ccclass, property } = cc._decorator;

@ccclass
export default class SceneComponent extends MyCocosSceneComponent {

    addPreloadImage() {
    }

    addPreloadAudio() {
    }

    addPreloadAnima() {

    }

    rows = null;
    orgId = 515;

    async onLoadEnd() {
        const middleLayer = cc.find('middleLayer');
        if (middleLayer) {
            const middleLayerComponent = middleLayer.getComponent('middleLayer');
            this.orgId = middleLayerComponent.ORG_ID;
        }

        await this.initData();
        await this.initView();
        this.initListener();
    }

    async loadData(pid = null) {
        const query = { orgid: this.orgId };
        if (pid) {
            query["pid"] = pid;
        }
		const res: any = await asyncCallNetworkApiGet("/api/syllabus/v1/list", query);
        let rows = JSON.parse(res).rows;

        const middleLayer = cc.find('middleLayer');
        if (middleLayer) {
            const syllabuses = cc.sys.localStorage.getItem('activeSyllabuses') || '';
            rows = rows.filter(item => (syllabuses=="all" || syllabuses.split(",").includes(item.id+"")));
        }
		return rows;
	}

    async initData() {

        const routes = this.getRouters();
        const route = (routes&&routes.length>0) ? routes[routes.length-1] : null;
        if (route) {
            cc.find(`Canvas/lefttop/bg_title/title`).getComponent(cc.Label).string = route.name;
            cc.find(`Canvas/lefttop`).active = true;
            cc.find(`Canvas/close`).active = false;
            this.rows = await this.loadData(route.id);
        } else {
            this.rows = await this.loadData();
        }
    }

    mockRouter = [];
    pushRouter (item) {
        const middleLayer = cc.find('middleLayer');
        if (!middleLayer) {
            this.mockRouter.push(item);
            return;
        }
        const middleLayerComponent = middleLayer.getComponent('middleLayer');
        middleLayerComponent.routers.push(item);
    }

    popRouter () {
        const middleLayer = cc.find('middleLayer');
        if (!middleLayer) {
            return this.mockRouter.pop();
        }
        const middleLayerComponent = middleLayer.getComponent('middleLayer');
        const item = middleLayerComponent.routers.pop();
        return item;
    }

    getRouters () {
        const middleLayer = cc.find('middleLayer');
        if (!middleLayer) {
            return this.mockRouter;
        }
        const middleLayerComponent = middleLayer.getComponent('middleLayer');
        return middleLayerComponent.routers;
    }

    async drawPage() {

        const contentNode = cc.find("Canvas/pages/view/content");
        contentNode.active = false;
        const item = cc.find("item", contentNode);
        // 清理原来的数据
		item.parent = this.node;
		contentNode.removeAllChildren();
        item.parent = contentNode;
        item.active = false;
        
        // 绘制
        for (let i = 0; i < this.rows.length; i++) {
            const row = this.rows[i];
            const itemClone = cc.instantiate(item);
			itemClone.parent = contentNode;
			itemClone.active = true;
			itemClone.attr({item_id: row.id, item_name: row.name});
            cc.find(`text`, itemClone).getComponent(cc.Label).string = row.name;
            if (row.cover) {
                const spriteFrame:any = await asyncGetSpriteFrimeByUrl(row.cover);
                const coverNode = cc.find(`cover`, itemClone);
                coverNode.getComponent(cc.Sprite).spriteFrame = spriteFrame;
                const sx = 340 / coverNode.width;
                const sy = 380 / coverNode.height;
                coverNode.scale = Math.min(sx, sy);
            }

            buttonOnClick(itemClone, async () => {
                if (row.has_courseware) {
                    // 打开课件
                    showLoading();
                    loadOnlineCourseWare(row.id);
                } else {
                    showLoading();
                    // 打开文件夹
                    const rows = await this.loadData(row.id);
                    if (rows.length > 0) {
                        this.rows = rows;
                        this.pushRouter(row);
                        await this.drawPage();
                        cc.find(`Canvas/lefttop/bg_title/title`).getComponent(cc.Label).string = row.name;
                        cc.find(`Canvas/lefttop`).active = true;
                        cc.find(`Canvas/close`).active = false;
                    } else {
                        showTips("内容还未制作！");
                    }
                    hideLoading();
                }
            });
        }

        contentNode.active = true;
        cc.find(`Canvas/pages`).getComponent(cc.ScrollView).scrollToTop(0.1);
    }

    async initView() {
        this.initBg();
        await this.drawPage();
    }

    initBg() {
        const bgNode = cc.find('Canvas/bg');
        bgNode.scale = this._mapScaleMax;
    }

    initListener() {
        const back = cc.find(`Canvas/lefttop/back`);
        buttonOnClick(back, async () => {
            showLoading();
            const item = this.popRouter();
            this.rows = await this.loadData(item.pid);
            await this.drawPage();
            hideLoading();

            const routers = this.getRouters();
            if (routers.length === 0) {
                cc.find(`Canvas/lefttop`).active = false;
                cc.find(`Canvas/close`).active = true;
            } else {
                cc.find(`Canvas/lefttop/bg_title/title`).getComponent(cc.Label).string = routers[routers.length-1].name;
            }
        });

        buttonOnClick(cc.find(`Canvas/close`), () => {
            if (window && (<any>window).courseware && (<any>window).courseware.exitApp) {
                (<any>window).courseware.exitApp();
            }
        });
    }

    playLocalAudio(audioName) {
        const audio = cc.find(`Canvas/res/audio/${audioName}`).getComponent(cc.AudioSource);
        return new Promise((resolve, reject) => {
            const id = cc.audioEngine.playEffect(audio.clip, false);
            cc.audioEngine.setFinishCallback(id, () => {
                resolve(id);
            });
        })
    }
}
